package org.kantega.jexmec.test;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.PluginClassLoaderProvider;
import org.kantega.jexmec.PluginLoader;
import org.kantega.jexmec.events.PluginRegistrationEvent;
import org.kantega.jexmec.manager.DefaultPluginManager;
import org.kantega.jexmec.events.PluginClassLoaderEvent;
import org.kantega.jexmec.events.PluginLoadingExceptionEvent;
import org.kantega.jexmec.events.PluginManagerEvent;

import java.util.List;

/**
 *
 */
public class TestEvents {

    public static class TestPluginManagerEvent<P> implements PluginManagerEvent<P> {
        private final DefaultPluginManager<P> pluginManger;

        public TestPluginManagerEvent(DefaultPluginManager<P> pluginManger) {
            this.pluginManger = pluginManger;
        }

        public DefaultPluginManager<P> getPluginManager() {
            return pluginManger;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || !PluginManagerEvent.class.isAssignableFrom(o.getClass()))
                return false;
            PluginManagerEvent that = (PluginManagerEvent) o;

            if (pluginManger != null ? !pluginManger.equals(that.getPluginManager()) : that.getPluginManager() != null)
                return false;

            return true;
        }

        @Override
        public int hashCode() {
            return pluginManger != null ? pluginManger.hashCode() : 0;
        }
    }

    public static class TestPluginClassLoaderEvent<P> extends TestPluginManagerEvent<P> implements PluginClassLoaderEvent<P> {
        private final PluginClassLoaderProvider provider;
        private final ClassLoader classLoader;

        public TestPluginClassLoaderEvent(DefaultPluginManager<P> pluginManger, PluginClassLoaderProvider provider, ClassLoader classLoader) {
            super(pluginManger);
            this.provider = provider;
            this.classLoader = classLoader;
        }


        public PluginClassLoaderProvider getProvider() {
            return provider;
        }

        public ClassLoader getClassLoader() {
            return classLoader;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || !PluginClassLoaderEvent.class.isAssignableFrom(o.getClass()))
                return false;

            PluginClassLoaderEvent that = (PluginClassLoaderEvent) o;

            if (classLoader != null ? !classLoader.equals(that.getClassLoader()) : that.getClassLoader() != null)
                return false;
            if (provider != null ? !provider.equals(that.getProvider()) : that.getProvider() != null) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = provider != null ? provider.hashCode() : 0;
            result = 31 * result + (classLoader != null ? classLoader.hashCode() : 0);
            return result;
        }
    }

    public static class TestPluginRegistrationEvent<P> extends TestPluginClassLoaderEvent<P> implements PluginRegistrationEvent<P> {

        private final List<P> plugins;
        private PluginLoader<P> pluginLoader;

        public TestPluginRegistrationEvent(DefaultPluginManager<P> pluginManger, PluginClassLoaderProvider provider, ClassLoader classLoader, PluginLoader<P> pluginLoader, List<P> plugins) {
            super(pluginManger, provider, classLoader);
            this.plugins = plugins;
            this.pluginLoader = pluginLoader;
        }

        public List<P> getPlugins() {
            return this.plugins;
        }

        public PluginLoader<P> getPluginLoader() {
            return pluginLoader;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || ! PluginRegistrationEvent.class.isAssignableFrom(o.getClass())) return false;
            if (!super.equals(o)) return false;

            PluginRegistrationEvent that = (PluginRegistrationEvent) o;

            if (!pluginLoader.equals(that.getPluginLoader())) return false;
            if (!plugins.equals(that.getPlugins())) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = super.hashCode();
            result = 31 * result + plugins.hashCode();
            result = 31 * result + pluginLoader.hashCode();
            return result;
        }
    }

    public static class TestPluginLoadingExceptionEvent<P> extends TestPluginClassLoaderEvent<P> implements PluginLoadingExceptionEvent<P> {
        private final PluginLoader<P> pluginLoader;
        private final Throwable throwable;

        public TestPluginLoadingExceptionEvent(DefaultPluginManager<P> pluginManger, PluginClassLoaderProvider provider, ClassLoader classLoader, PluginLoader<P> pluginLoader, Throwable throwable) {
            super(pluginManger, provider, classLoader);
            this.pluginLoader = pluginLoader;
            this.throwable = throwable;
        }

        public PluginLoader<P> getPluginLoader() {
            return pluginLoader;
        }

        public Throwable getThrowable() {
            return throwable;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || !PluginLoadingExceptionEvent.class.isAssignableFrom(o.getClass()))
                return false;

            PluginLoadingExceptionEvent that = (PluginLoadingExceptionEvent) o;

            if (pluginLoader != null ? !pluginLoader.equals(that.getPluginLoader()) : that.getPluginLoader() != null)
                return false;
            if (throwable != null ? !throwable.equals(that.getThrowable()) : that.getThrowable() != null) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = super.hashCode();
            result = 31 * result + (pluginLoader != null ? pluginLoader.hashCode() : 0);
            result = 31 * result + (throwable != null ? throwable.hashCode() : 0);
            return result;
        }
    }
}
